==> JVPC-0.93.zip: JVPC-0.93/vga/MDataGen.java <==
// File: MdataGen.java

package jvpc.vga;

import java.io.*;

/**
 * Generates messages; opens file ``mdata??.dat'' and reads incoming messages.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     12 II 1999
 * @version   21 XII 1999
 */
public class MDataGen extends Generator {
  private int mdatas;
  OutputFileOpener ofo;

  public MDataGen(InputStream is) throws IOException {
    super(is);
    mdatas=source.readShort();
  }

  public MDataGen(OutputFileOpener o) {
    super(); // waarom dan?
    ofo=o;
  }

  public MDataGen(Generator g) throws IOException {
    super(g);
    mdatas=source.readShort();
  }

  public void generateAll(Universe u,int player,int offset) {
    int i;
    int[] pointer=new int[mdatas];
    int[] length=new int[mdatas];
    // System.err.println("Reading "+mdatas+" messages... ");
    try {
      // current ``posistion'' is offset + 0
      for (i=0;i<mdatas;i++) {
	pointer[i]=source.readInt()-1;
	length[i]=source.readShort();
	// System.err.print("@"+pointer[i]+"["+length[i]+"]");
      }
      // current ``position'' is offset + 2 + mdatas * (4+2)
      int curpos=offset+2+mdatas*6;
      for (i=0;i<mdatas;i++) {
	if (curpos<pointer[i]) {
	  source.skip(pointer[i]-curpos);
	  curpos=pointer[i];
	}
	Message mdata=new Message(u);
	mdata.readMDataFrom(source,length[i],player);
	curpos+=length[i]; // only contents
	u.putMessage(mdata);
      }
    } catch (IOException ioe) {
      crash(11, "Could not read mdata, probably truncated file");
    }
  }

  /**
   * Write all incoming messages to file.
   */
  public void saveMData(Universe u) throws IOException {
    OutputStream outfile;
    LSBWriter[] lsbw=new LSBWriter[races+1];
    int[] count=new int[races+1];
    int[] pointer=new int[races+1];
    //int[] length=new int[races];
    int i,player;

    // count messages per player
    for (i=0;i<u.getMessageCount();i++) {
      Message message=u.getMessage(i);
      int owner=message.getRecipient();
      count[owner]++;
    }

    // write pointers and lengths
    for (i=0;i<u.getMessageCount();i++) {
      Message message=u.getMessage(i);
      int owner=message.getRecipient();
      if (lsbw[owner]==null) {
	outfile=ofo.open("mdata"+owner+".dat");
	lsbw[owner]=new LSBWriter(outfile);
	lsbw[owner].writeShort((short)count[owner]);
	pointer[owner]=2+6*count[owner]+1;
      }
      lsbw[owner].writeInt(pointer[owner]);
      lsbw[owner].writeShort((short)message.getLength());
      pointer[owner]+=message.getLength();
    }

    // write contents
    for (i=0;i<u.getMessageCount();i++) {
      Message message=u.getMessage(i);
      int owner=message.getRecipient();
      try {
	u.getMessage(i).writeMessageTo(lsbw[owner]);
      } catch (IOException ioe) {
	// hubba
	crash(11, "Failed to write message "+i+" to file; "+ioe);
      }
    }

    // no signature needed, just close file
    for (i=0;i<races;i++) {
      player=(short)(i+1);
      if (lsbw[player]!=null) try {
	lsbw[player].close();
      } catch (Exception ioe) {
	crash(11, "Failed to finish incoming??.dat: "+ioe);
      }
    }
  }
}

==> JVPC-0.94.zip: JVPC-0.94/vga/MDataGen.java <==
// File: MdataGen.java

package jvpc.vga;

import java.io.*;

/**
 * Generates messages; opens file ``mdata??.dat'' and reads incoming messages.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     12 II 1999
 * @version   21 XII 1999
 */
public class MDataGen extends Generator {
  private int mdatas;
  OutputFileOpener ofo;

  public MDataGen(InputStream is) throws IOException {
    super(is);
    mdatas=source.readShort();
  }

  public MDataGen(OutputFileOpener o) {
    super(); // waarom dan?
    ofo=o;
  }

  public MDataGen(Generator g) throws IOException {
    super(g);
    mdatas=source.readShort();
  }

  public void generateAll(Universe u,int player,int offset) {
    int i;
    int[] pointer=new int[mdatas];
    int[] length=new int[mdatas];
    // System.err.println("Reading "+mdatas+" messages... ");
    try {
      // current ``posistion'' is offset + 0
      for (i=0;i<mdatas;i++) {
	pointer[i]=source.readInt()-1;
	length[i]=source.readShort();
	// System.err.print("@"+pointer[i]+"["+length[i]+"]");
      }
      // current ``position'' is offset + 2 + mdatas * (4+2)
      int curpos=offset+2+mdatas*6;
      for (i=0;i<mdatas;i++) {
	if (curpos<pointer[i]) {
	  source.skip(pointer[i]-curpos);
	  curpos=pointer[i];
	}
	Message mdata=new Message(u);
	mdata.readMDataFrom(source,length[i],u,player);
	curpos+=length[i]; // only contents
	u.putMessage(mdata);
      }
    } catch (IOException ioe) {
      crash(11, "Could not read mdata, probably truncated file");
    }
  }

  /**
   * Write all incoming messages to file.
   */
  public void saveMData(Universe u) throws IOException {
    OutputStream outfile;
    LSBWriter[] lsbw=new LSBWriter[races+1];
    int[] count=new int[races+1];
    int[] pointer=new int[races+1];
    //int[] length=new int[races];
    int i,player;

    // count messages per player
    for (i=0;i<u.getMessageCount();i++) {
      Message message=u.getMessage(i);
      int owner=message.getRecipient();
      count[owner]++;
    }

    // write pointers and lengths
    for (i=0;i<u.getMessageCount();i++) {
      Message message=u.getMessage(i);
      int owner=message.getRecipient();
      if (lsbw[owner]==null) {
	outfile=ofo.open("mdata"+owner+".dat");
	lsbw[owner]=new LSBWriter(outfile);
	lsbw[owner].writeShort((short)count[owner]);
	pointer[owner]=2+6*count[owner]+1;
      }
      lsbw[owner].writeInt(pointer[owner]);
      lsbw[owner].writeShort((short)message.getLength());
      pointer[owner]+=message.getLength();
    }

    // write contents
    for (i=0;i<u.getMessageCount();i++) {
      Message message=u.getMessage(i);
      int owner=message.getRecipient();
      try {
	u.getMessage(i).writeMessageTo(lsbw[owner]);
      } catch (IOException ioe) {
	// hubba
	crash(11, "Failed to write message "+i+" to file; "+ioe);
      }
    }

    // no signature needed, just close file
    for (i=0;i<races;i++) {
      player=(short)(i+1);
      if (lsbw[player]!=null) try {
	lsbw[player].close();
      } catch (Exception ioe) {
	crash(11, "Failed to finish incoming??.dat: "+ioe);
      }
    }
  }
}
